// HullLine.cpp: implementation of the CHullLine class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ZiyooHull.h"
#include "HullLine.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHullLine::CHullLine()
{
    PtNum = 0;
}

CHullLine::~CHullLine()
{
    CHullPoint *pPt;
    for( int i = 0; i < PtNum; i++)
    {
        pPt = PtArray[i];
        delete pPt;
    }

    PtArray.RemoveAll();
}

void CHullLine::DrawLine(BOOL IsLoop)
{
    int i, size;
    GLushort  pattern;
    int factor;

    factor =1;
    pattern = 0x5555;
    size = HPtArray.GetSize();

    glColor4d(0, 0, 0, 0.5);
    glLineWidth(2.0);

    if( size > 0 ) 
    {
        glEnable(GL_LINE_STIPPLE);
        glLineStipple(factor, pattern);
    }
    else
    {
        glDisable(GL_LINE_STIPPLE);
    }

    if( IsLoop )
        glBegin(GL_LINE_LOOP);
    else
        glBegin(GL_LINE_STRIP);
    for( i = 0; i < PtNum; i++ )
    {
        glVertex2d(PtArray[i]->x,PtArray[i]->y);
    }
    glEnd();
    
    glColor4d(1.0, 0, 0, 0.5);

    glDisable(GL_LINE_STIPPLE);
    glBegin(GL_LINE_LOOP);
    for( i = 0; i < size; i++ )
    {
        glVertex2d(HPtArray[i]->x,HPtArray[i]->y);
    }
    glEnd();

    glLineWidth(1.0);
}

void CHullLine::AddPoint(double x, double y)
{
    CHullPoint *pPt;

    pPt = new CHullPoint;
    
    pPt->x = x;
    pPt->y = y;

    PtArray.Add(pPt);

    PtNum++;
}

int CHullLine::ConvexHull2D(CHullPoint P[], CHullPoint H[], int n)
{
// the output array H[] will be used as the stack
    int    bot, top;  // indices for bottom and top of the stack
    int    i;                // array scan index

    // Get the indices of points with min x-coord and min|max y-coord
    int minmin, minmax;
    double xmin;
    // Get the indices of points with max x-coord and min|max y-coord
    int maxmin, maxmax;
    double xmax;

    bot = 0;
    top = -1;
    minmin = 0;
    xmin = P[0].x;

    for( i = 1; i < n; i++ )
    {
        if(P[i].x != xmin) break;
    }

    minmax = i-1;
    
    if( minmax == n-1 )// degenerate case: all x-coords == xmin
    {       
        H[++top] = P[minmin];

        if(P[minmax].y != P[minmin].y) // a nontrivial segment
            H[++top] = P[minmax];

        H[++top] = P[minmin];           // add polygon endpoint

        return top+1;
    }


    maxmax = n-1;
    xmax = P[n-1].x;

    for( i = n-2; i >= 0; i-- )
    {
        if( P[i].x != xmax ) break;
    }

    maxmin = i+1;

    // Compute the lower hull on the stack H
    H[++top] = P[minmin];      // push minmin point onto stack
    i = minmax;
    while (++i <= maxmin)
    {
        // the lower line joins P[minmin] with P[maxmin]
        if (IsPoint2DLeft( P[minmin], P[maxmin], P[i]) >= 0 && i < maxmin)
            continue;          // ignore P[i] above or on the lower line

        while (top > 0)        // there are at least 2 points on the stack
        {
            // test if P[i] is left of the line at the stack top
            if (IsPoint2DLeft( H[top-1], H[top], P[i]) > 0)
                break;         // P[i] is a new hull vertex
            else
                top--;         // pop top point off stack
        }
        H[++top] = P[i];       // push P[i] onto stack
    }

    // Next, compute the upper hull on the stack H above the bottom hull
    if (maxmax != maxmin)      // if distinct xmax points
        H[++top] = P[maxmax];  // push maxmax point onto stack

    bot = top;                 // the bottom point of the upper hull stack
    i = maxmin;
    while (--i >= minmax)
    {
        // the upper line joins P[maxmax] with P[minmax]
        if (IsPoint2DLeft( P[maxmax], P[minmax], P[i]) >= 0 && i > minmax)
            continue;          // ignore P[i] below or on the upper line

        while (top > bot)    // at least 2 points on the upper stack
        {
            // test if P[i] is left of the line at the stack top
            if (IsPoint2DLeft( H[top-1], H[top], P[i]) > 0)
                break;         // P[i] is a new hull vertex
            else
                top--;         // pop top point off stack
        }
        H[++top] = P[i];       // push P[i] onto stack
    }
    if (minmax != minmin)
        H[++top] = P[minmin];  // push joining endpoint onto stack

    return top+1;
}

double CHullLine::IsPoint2DLeft(CHullPoint P0, CHullPoint P1, CHullPoint P2)
{
   return (P1.x - P0.x)*(P2.y - P0.y) - (P2.x - P0.x)*(P1.y - P0.y);
}

void CHullLine::CalculateConvexHull()
{
    int i, HullPtNum;
    CHullPoint InputPt[1000], OutputPt[1000], *pPt;

    ResetHPtArray();

    for( i = 0; i < PtNum; i++)
    {
        InputPt[i].x = PtArray[i]->x;
        InputPt[i].y = PtArray[i]->y;
    }

    QuickSortPtsBasedOnX(InputPt, PtNum);

    HullPtNum = ConvexHull2D(InputPt, OutputPt, PtNum);

    for( i = 0; i < HullPtNum; i++)
    {
        pPt = new CHullPoint;
    
        pPt->x = OutputPt[i].x;
        pPt->y = OutputPt[i].y;

        HPtArray.Add(pPt);
    }
}

void CHullLine::QuickSortPtsBasedOnX(CHullPoint InputPt[], int Num)
{
    int i, j;
    int MinIndex;
    double tempx;

    for( i = 0; i < Num-1; i++)
    {
        MinIndex = i;
        for( j = i+1; j < Num; j++)
        {
            if( InputPt[j].x < InputPt[MinIndex].x )
            {
                tempx = InputPt[j].x;
                InputPt[j].x = InputPt[MinIndex].x;
                InputPt[MinIndex].x = tempx;
                
                tempx = InputPt[j].y;
                InputPt[j].y = InputPt[MinIndex].y;
                InputPt[MinIndex].y = tempx;
            }
        }
    }
}

void CHullLine::ResetHPtArray()
{
    int i, size;
    CHullPoint *pt;

    size = HPtArray.GetSize();

    for( i = 0; i < size; i++ )
    {
        pt = HPtArray[i];
        delete pt;
    }

    HPtArray.RemoveAll();
}
